@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: formdata.c,v 1.29 2002/01/19 11:08:05 bagder Exp $
+ * $Id: formdata.c,v 1.36 2002/04/03 11:11:01 bagder Exp $
  *****************************************************************************/
 
 /*
@@ -171,8 +171,8 @@ static void GetStr(char **string,
 
 static
 int FormParse(char *input,
-	      struct HttpPost **httppost,
-	      struct HttpPost **last_post)
+	      struct curl_httppost **httppost,
+	      struct curl_httppost **last_post)
 {
   /* nextarg MUST be a string in the format 'name=contents' and we'll
      build a linked list with the info */
@@ -186,8 +186,8 @@ int FormParse(char *input,
   char *prevtype = NULL;
   char *sep;
   char *sep2;
-  struct HttpPost *post;
-  struct HttpPost *subpost; /* a sub-node */
+  struct curl_httppost *post;
+  struct curl_httppost *subpost; /* a sub-node */
   unsigned int i;
 
   /* Preallocate contents to the length of input to make sure we don't
@@ -235,7 +235,6 @@ int FormParse(char *input,
 	    
 	    if(2 != sscanf(type, "%127[^/]/%127[^,\n]",
 			   major, minor)) {
-	      fprintf(stderr, "Illegally formatted content-type field!\n");
               free(contents);
 	      return 2; /* illegal content-type syntax! */
 	    }
@@ -297,9 +296,9 @@ int FormParse(char *input,
 	  /* For the first file name, we allocate and initiate the main list
 	     node */
 
-	  post = (struct HttpPost *)malloc(sizeof(struct HttpPost));
+	  post = (struct curl_httppost *)malloc(sizeof(struct curl_httppost));
 	  if(post) {
-	    memset(post, 0, sizeof(struct HttpPost));
+	    memset(post, 0, sizeof(struct curl_httppost));
 	    GetStr(&post->name, name);      /* get the name */
 	    GetStr(&post->contents, contp); /* get the contents */
             post->contentslength = 0;
@@ -321,9 +320,10 @@ int FormParse(char *input,
 	else {
 	  /* we add a file name to the previously allocated node, known as
              'post' now */
-	  subpost =(struct HttpPost *)malloc(sizeof(struct HttpPost));
+	  subpost =(struct curl_httppost *)
+            malloc(sizeof(struct curl_httppost));
 	  if(subpost) {
-	     memset(subpost, 0, sizeof(struct HttpPost));
+	     memset(subpost, 0, sizeof(struct curl_httppost));
 	     GetStr(&subpost->name, name);      /* get the name */
 	     GetStr(&subpost->contents, contp); /* get the contents */
              subpost->contentslength = 0;
@@ -343,9 +343,9 @@ int FormParse(char *input,
       } while(sep && *sep); /* loop if there's another file name */
     }
     else {
-      post = (struct HttpPost *)malloc(sizeof(struct HttpPost));
+      post = (struct curl_httppost *)malloc(sizeof(struct curl_httppost));
       if(post) {
-	memset(post, 0, sizeof(struct HttpPost));
+	memset(post, 0, sizeof(struct curl_httppost));
 	GetStr(&post->name, name);      /* get the name */
 	if( contp[0]=='<' ) {
 	  GetStr(&post->contents, contp+1); /* get the contents */
@@ -371,7 +371,6 @@ int FormParse(char *input,
 
   }
   else {
-    fprintf(stderr, "Illegally formatted input field!\n");
     free(contents);
     return 1;
   }
@@ -380,8 +379,8 @@ int FormParse(char *input,
 }
 
 int curl_formparse(char *input,
-                   struct HttpPost **httppost,
-                   struct HttpPost **last_post)
+                   struct curl_httppost **httppost,
+                   struct curl_httppost **last_post)
 {
   return FormParse(input, httppost, last_post);
 }
@@ -396,27 +395,28 @@ int curl_formparse(char *input,
  * Returns newly allocated HttpPost on success and NULL if malloc failed.
  *
  ***************************************************************************/
-static struct HttpPost * AddHttpPost(char * name,
-                                     long namelength,
-                                     char * value,
-                                     long contentslength,
-                                     char *contenttype,
-                                     long flags,
-                                     struct curl_slist* contentHeader,
-                                     struct HttpPost *parent_post,
-                                     struct HttpPost **httppost,
-                                     struct HttpPost **last_post)
+static struct curl_httppost *
+AddHttpPost(char * name, long namelength,
+            char * value, long contentslength,
+            char *contenttype,
+            long flags,
+            struct curl_slist* contentHeader,
+            char *showfilename,
+            struct curl_httppost *parent_post,
+            struct curl_httppost **httppost,
+            struct curl_httppost **last_post)
 {
-  struct HttpPost *post;
-  post = (struct HttpPost *)malloc(sizeof(struct HttpPost));
+  struct curl_httppost *post;
+  post = (struct curl_httppost *)malloc(sizeof(struct curl_httppost));
   if(post) {
-    memset(post, 0, sizeof(struct HttpPost));
+    memset(post, 0, sizeof(struct curl_httppost));
     post->name = name;
     post->namelength = name?(namelength?namelength:(long)strlen(name)):0;
     post->contents = value;
     post->contentslength = contentslength;
     post->contenttype = contenttype;
     post->contentheader = contentHeader;
+    post->showfilename = showfilename;
     post->flags = flags;
   }
   else
@@ -629,17 +629,17 @@ typedef enum {
 } FORMcode;
 
 static
-FORMcode FormAdd(struct HttpPost **httppost,
-                 struct HttpPost **last_post,
+FORMcode FormAdd(struct curl_httppost **httppost,
+                 struct curl_httppost **last_post,
                  va_list params)
 {
   FormInfo *first_form, *current_form, *form;
   FORMcode return_value = FORMADD_OK;
   const char *prevtype = NULL;
-  struct HttpPost *post = NULL;
+  struct curl_httppost *post = NULL;
   CURLformoption option;
   struct curl_forms *forms = NULL;
-  const char *array_value; /* value read from an array */
+  char *array_value; /* value read from an array */
 
   /* This is a state variable, that if TRUE means that we're parsing an
      array that we got passed to us. If FALSE we're parsing the input
@@ -670,7 +670,7 @@ FORMcode FormAdd(struct HttpPost **httppost,
     if ( array_state ) {
       /* get the upcoming option from the given array */
       option = forms->option;
-      array_value = forms->value;
+      array_value = (char *)forms->value;
 
       forms++; /* advance this to next entry */
       if (CURLFORM_END == option) {
@@ -678,16 +678,6 @@ FORMcode FormAdd(struct HttpPost **httppost,
         array_state = FALSE;
         continue;
       }
-      else {
-        /* check that the option is OK in an array */
-
-        /* Daniel's note: do we really need to do this? */
-        if ( (option <= CURLFORM_ARRAY_START) ||
-             (option >= CURLFORM_ARRAY_END) ) {
-          return_value = FORMADD_ILLEGAL_ARRAY;
-          break;
-        }
-      }
     }
     else {
       /* This is not array-state, get next option */
@@ -698,11 +688,16 @@ FORMcode FormAdd(struct HttpPost **httppost,
 
     switch (option) {
     case CURLFORM_ARRAY:
-      forms = va_arg(params, struct curl_forms *);
-      if (forms)
-        array_state = TRUE;
-      else
-        return_value = FORMADD_NULL;
+      if(array_state)
+        /* we don't support an array from within an array */
+        return_value = FORMADD_ILLEGAL_ARRAY;
+      else {
+        forms = va_arg(params, struct curl_forms *);
+        if (forms)
+          array_state = TRUE;
+        else
+          return_value = FORMADD_NULL;
+      }
       break;
 
       /*
@@ -714,7 +709,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
       if (current_form->name)
         return_value = FORMADD_OPTION_TWICE;
       else {
-        char *name = va_arg(params, char *);
+        char *name = array_state?
+          array_value:va_arg(params, char *);
         if (name)
           current_form->name = name; /* store for the moment */
         else
@@ -725,7 +721,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
       if (current_form->namelength)
         return_value = FORMADD_OPTION_TWICE;
       else
-        current_form->namelength = va_arg(params, long);
+        current_form->namelength =
+          array_state?(long)array_value:va_arg(params, long);
       break;
 
       /*
@@ -737,7 +734,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
       if (current_form->value)
         return_value = FORMADD_OPTION_TWICE;
       else {
-        char *value = va_arg(params, char *);
+        char *value =
+          array_state?array_value:va_arg(params, char *);
         if (value)
           current_form->value = value; /* store for the moment */
         else
@@ -748,7 +746,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
       if (current_form->contentslength)
         return_value = FORMADD_OPTION_TWICE;
       else
-        current_form->contentslength = va_arg(params, long);
+        current_form->contentslength =
+          array_state?(long)array_value:va_arg(params, long);
       break;
 
       /* Get contents from a given file name */
@@ -756,7 +755,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
       if (current_form->flags != 0)
         return_value = FORMADD_OPTION_TWICE;
       else {
-        char *filename = va_arg(params, char *);
+        char *filename = array_state?
+          array_value:va_arg(params, char *);
         if (filename) {
           current_form->value = strdup(filename);
           current_form->flags |= HTTPPOST_READFILE;
@@ -769,11 +769,9 @@ FORMcode FormAdd(struct HttpPost **httppost,
       /* We upload a file */
     case CURLFORM_FILE:
       {
-        const char *filename = NULL;
-        if (array_state)
-          filename = array_value;
-        else
-          filename = va_arg(params, const char *);
+        char *filename = array_state?array_value:
+          va_arg(params, char *);
+
         if (current_form->value) {
           if (current_form->flags & HTTPPOST_FILENAME) {
             if (filename) {
@@ -798,11 +796,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
       }
     case CURLFORM_CONTENTTYPE:
       {
-	const char *contenttype = NULL;
-        if (array_state)
-          contenttype = array_value;
-        else
-          contenttype = va_arg(params, const char *);
+	char *contenttype =
+          array_state?array_value:va_arg(params, char *);
         if (current_form->contenttype) {
           if (current_form->flags & HTTPPOST_FILENAME) {
             if (contenttype) {
@@ -827,11 +822,11 @@ FORMcode FormAdd(struct HttpPost **httppost,
       }
     case CURLFORM_CONTENTHEADER:
       {
-        struct curl_slist* list = NULL;
-        if( array_state )
-          list = (struct curl_slist*)array_value;
-        else
-          list = va_arg(params,struct curl_slist*);
+        /* this "cast increases required alignment of target type" but
+           we consider it OK anyway */
+        struct curl_slist* list = array_state?
+          (struct curl_slist*)array_value:
+          va_arg(params, struct curl_slist*);
         
         if( current_form->contentheader )
           return_value = FORMADD_OPTION_TWICE;
@@ -840,8 +835,17 @@ FORMcode FormAdd(struct HttpPost **httppost,
         
         break;
       }
+    case CURLFORM_FILENAME:
+      {
+        char *filename = array_state?array_value:
+          va_arg(params, char *);
+        if( current_form->showfilename )
+          return_value = FORMADD_OPTION_TWICE;
+        else
+          current_form->showfilename = strdup(filename);
+        break;
+      }
     default:
-      fprintf (stderr, "got unknown CURLFORM_OPTION: %d\n", option);
       return_value = FORMADD_UNKNOWN_OPTION;
     }
   }
@@ -892,7 +896,7 @@ FORMcode FormAdd(struct HttpPost **httppost,
         post = AddHttpPost(form->name, form->namelength,
                            form->value, form->contentslength,
                            form->contenttype, form->flags,
-                           form->contentheader,
+                           form->contentheader, form->showfilename,
                            post, httppost,
                            last_post);
         
@@ -918,8 +922,8 @@ FORMcode FormAdd(struct HttpPost **httppost,
   return return_value;
 }
 
-int curl_formadd(struct HttpPost **httppost,
-                 struct HttpPost **last_post,
+int curl_formadd(struct curl_httppost **httppost,
+                 struct curl_httppost **last_post,
                  ...)
 {
   va_list arg;
@@ -978,8 +982,8 @@ char *Curl_FormBoundary(void)
 			      the same form won't be identical */
   int i;
 
-  static char table64[]=
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+  static char table62[]=
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
 
   retstring = (char *)malloc(BOUNDARY_LENGTH);
 
@@ -991,7 +995,7 @@ char *Curl_FormBoundary(void)
   strcpy(retstring, "curl"); /* bonus commercials 8*) */
 
   for(i=4; i<(BOUNDARY_LENGTH-1); i++) {
-    retstring[i] = table64[rand()%64];
+    retstring[i] = table62[rand()%62];
   }
   retstring[BOUNDARY_LENGTH-1]=0; /* zero terminate */
 
@@ -1012,9 +1016,9 @@ void Curl_formclean(struct FormData *form)
 }
 
 /* external function to free up a whole form post chain */
-void curl_formfree(struct HttpPost *form)
+void curl_formfree(struct curl_httppost *form)
 {
-  struct HttpPost *next;
+  struct curl_httppost *next;
 
   if(!form)
     /* no form to free, just get out of this */
@@ -1033,18 +1037,20 @@ void curl_formfree(struct HttpPost *form)
       free(form->contents); /* free the contents */
     if(form->contenttype)
       free(form->contenttype); /* free the content type */
+    if(form->showfilename)
+      free(form->showfilename); /* free the faked file name */
     free(form);       /* free the struct */
 
   } while((form=next)); /* continue */
 }
 
-struct FormData *Curl_getFormData(struct HttpPost *post,
+struct FormData *Curl_getFormData(struct curl_httppost *post,
                                   int *sizep)
 {
   struct FormData *form = NULL;
   struct FormData *firstform;
 
-  struct HttpPost *file;
+  struct curl_httppost *file;
 
   int size =0;
   char *boundary;
@@ -1068,8 +1074,11 @@ struct FormData *Curl_getFormData(struct HttpPost *post,
   
   do {
 
+    if(size)
+      size += AddFormDataf(&form, "\r\n");
+
     /* boundary */
-    size += AddFormDataf(&form, "\r\n--%s\r\n", boundary);
+    size += AddFormDataf(&form, "--%s\r\n", boundary);
 
     size += AddFormData(&form,
                         "Content-Disposition: form-data; name=\"", 0);
@@ -1093,16 +1102,25 @@ struct FormData *Curl_getFormData(struct HttpPost *post,
     file = post;
 
     do {
+
+      /* If 'showfilename' is set, that is a faked name passed on to us
+         to use to in the formpost. If that is not set, the actually used
+         local file name should be added. */
+
       if(post->more) {
 	/* if multiple-file */
 	size += AddFormDataf(&form,
-			     "\r\n--%s\r\nContent-Disposition: attachment; filename=\"%s\"",
-			     fileboundary, file->contents);
+			     "\r\n--%s\r\nContent-Disposition: "
+                             "attachment; filename=\"%s\"",
+			     fileboundary,
+                             (file->showfilename?file->showfilename:
+                              file->contents));
       }
       else if(post->flags & HTTPPOST_FILENAME) {
 	size += AddFormDataf(&form,
 			     "; filename=\"%s\"",
-			     post->contents);
+			     (post->showfilename?post->showfilename:
+                              post->contents));
       }
       
       if(file->contenttype) {
@@ -1294,8 +1312,8 @@ int Curl_FormReadOneLine(char *buffer,
 
 #ifdef _FORM_DEBUG
 int FormAddTest(const char * errormsg,
-                 struct HttpPost **httppost,
-                 struct HttpPost **last_post,
+                 struct curl_httppost **httppost,
+                 struct curl_httppost **last_post,
                  ...)
 {
   int result;
@@ -1341,8 +1359,8 @@ int main()
   int size;
   int nread;
   char buffer[4096];
-  struct HttpPost *httppost=NULL;
-  struct HttpPost *last_post=NULL;
+  struct curl_httppost *httppost=NULL;
+  struct curl_httppost *last_post=NULL;
   struct curl_forms forms[4];
 
   struct FormData *form;
@@ -1451,9 +1469,9 @@ int main(int argc, char **argv)
 #endif
   int i;
   char *nextarg;
-  struct HttpPost *httppost=NULL;
-  struct HttpPost *last_post=NULL;
-  struct HttpPost *post;
+  struct curl_httppost *httppost=NULL;
+  struct curl_httppost *last_post=NULL;
+  struct curl_httppost *post;
   int size;
   int nread;
   char buffer[4096];
