@@ -8,7 +8,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -21,7 +21,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: formdata.h,v 1.8 2001/08/28 08:54:33 bagder Exp $
+ * $Id: formdata.h,v 1.11 2002/03/19 07:54:55 bagder Exp $
  *****************************************************************************/
 /* plain and simple linked list with lines to send */
 struct FormData {
@@ -44,6 +44,10 @@ typedef struct FormInfo {
   long contentslength;
   char *contenttype;
   long flags;
+
+  char *showfilename; /* The file name to show. If not set, the actual
+                         file name will be used */
+  struct curl_slist* contentheader;
   struct FormInfo *more;
 } FormInfo;
 
