@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: timeval.c,v 1.7 2001/09/07 04:01:33 bumblebury Exp $
+ * $Id: timeval.c,v 1.12 2002/03/19 07:54:55 bagder Exp $
  *****************************************************************************/
 
 #ifdef WIN32
@@ -65,9 +65,14 @@ struct timeval Curl_tvnow (void)
  return now;
 }
 
-double Curl_tvdiff (struct timeval t1, struct timeval t2)
+/*
+ * Make sure that the first argument is the more recent time, as otherwise
+ * we'll get a weird negative time-diff back...
+ */
+long Curl_tvdiff (struct timeval newer, struct timeval older)
 {
- return (double)(t1.tv_sec - t2.tv_sec) + ((t1.tv_usec-t2.tv_usec)/1000000.0);
+  return (newer.tv_sec-older.tv_sec)*1000+
+    (499+newer.tv_usec-older.tv_usec)/1000;
 }
 
 long Curl_tvlong (struct timeval t1)
@@ -79,6 +84,6 @@ long Curl_tvlong (struct timeval t1)
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
