@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: transfer.c,v 1.78 2002/01/23 07:15:32 bagder Exp $
+ * $Id: transfer.c,v 1.91 2002/04/10 13:44:42 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -193,7 +193,6 @@ CURLcode Curl_readwrite(struct connectdata *conn,
       if ((k->bytecount == 0) && (k->writebytecount == 0))
         Curl_pgrsTime(data, TIMER_STARTTRANSFER);
 
-
       didwhat |= KEEP_READ;
 
       /* NULL terminate, allowing string ops to be used */
@@ -216,6 +215,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
          headers at the moment or not. */          
       if (k->header) {
         /* we are in parse-the-header-mode */
+        bool stop_reading = FALSE;
 
         /* header line within buffer loop */
         do {
@@ -244,7 +244,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
               newbuff = (char *)realloc(data->state.headerbuff, newsize);
               if(!newbuff) {
                 failf (data, "Failed to alloc memory for big header!");
-                return CURLE_READ_ERROR;
+                return CURLE_OUT_OF_MEMORY;
               }
               data->state.headersize=newsize;
               data->state.headerbuff = newbuff;
@@ -274,7 +274,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             newbuff = (char *)realloc(data->state.headerbuff, newsize);
             if(!newbuff) {
               failf (data, "Failed to alloc memory for big header!");
-              return CURLE_READ_ERROR;
+              return CURLE_OUT_OF_MEMORY;
             }
             data->state.headersize= newsize;
             data->state.headerbuff = newbuff;
@@ -313,6 +313,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
               k->headerline = 0; /* restart the header line counter */
               /* if we did wait for this do enable write now! */
               if (k->write_after_100_header) {
+
                 k->write_after_100_header = FALSE;
                 FD_SET (conn->writesockfd, &k->writefd); /* write */
                 k->keepon |= KEEP_WRITE;
@@ -356,8 +357,6 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                * If we requested a "no body", this is a good time to get
                * out and return home.
                */
-              bool stop_reading = FALSE;
-
               if(data->set.no_body)
                 stop_reading = TRUE;
               else if(!conn->bits.close) {
@@ -367,20 +366,16 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                    reading! */
                 if(-1 != conn->size)
                   conn->maxdownload = conn->size;
-
-                /* If max download size is *zero* (nothing) we already
-                   have nothing and can safely return ok now! */
-                if(0 == conn->maxdownload)
-                  stop_reading = TRUE;
-                    
-                /* What to do if the size is *not* known? */
               }
-
+              /* If max download size is *zero* (nothing) we already
+                 have nothing and can safely return ok now! */
+              if(0 == conn->maxdownload)
+                stop_reading = TRUE;
+                    
               if(stop_reading) {
                 /* we make sure that this socket isn't read more now */
                 k->keepon &= ~KEEP_READ;
                 FD_ZERO(&k->rkeepfd);
-                return CURLE_OK;
               }
 
               break;		/* exit header line loop */
@@ -449,6 +444,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                  * NOT contain a message-body, and thus is always terminated
                  * by the first empty line after the header fields.  */
                 conn->size=0;
+                conn->maxdownload=0;
                 break;
               default:
                 /* nothing */
@@ -466,6 +462,31 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             conn->size = k->contentlength;
             Curl_pgrsSetDownloadSize(data, k->contentlength);
           }
+          /* check for Content-Type: header lines to get the mime-type */
+          else if (strnequal("Content-Type:", k->p, 13)) {
+            char *start;
+            char *end;
+            int len;
+
+            /* Find the first non-space letter */
+            for(start=k->p+14;
+                *start && isspace((int)*start);
+                start++);
+
+            /* count all non-space letters following */
+            for(end=start, len=0;
+                *end && !isspace((int)*end);
+                end++, len++);
+
+            /* allocate memory of a cloned copy */
+            data->info.contenttype = malloc(len + 1);
+            if (NULL == data->info.contenttype)
+	      return CURLE_OUT_OF_MEMORY;
+
+            /* copy the content-type string */
+	    memcpy(data->info.contenttype, start, len);
+            data->info.contenttype[len] = 0; /* zero terminate */
+          }
           else if((k->httpversion == 10) &&
                   conn->bits.httpproxy &&
                   compareheader(k->p, "Proxy-Connection:", "keep-alive")) {
@@ -576,12 +597,17 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             return result;
 
           data->info.header_size += k->hbuflen;
+          conn->headerbytecount += k->hbuflen;
               
           /* reset hbufp pointer && hbuflen */
           k->hbufp = data->state.headerbuff;
           k->hbuflen = 0;
         }
-        while (*k->str); /* header line within buffer */
+        while (!stop_reading && *k->str); /* header line within buffer */
+
+        if(stop_reading)
+          /* We've stopped dealing with input, get out of the do-while loop */
+          break;
 
         /* We might have reached the end of the header part here, but
            there might be a non-header part left in the end of the read
@@ -610,6 +636,8 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             if (conn->newurl) {
               /* abort after the headers if "follow Location" is set */
               infof (data, "Follow to new URL: %s\n", conn->newurl);
+                k->keepon &= ~KEEP_READ;
+                FD_ZERO(&k->rkeepfd);
               return CURLE_OK;
             }
             else if (conn->resume_from &&
@@ -663,8 +691,12 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             Curl_httpchunk_read(conn, k->str, nread, &nread);
 
           if(CHUNKE_OK < res) {
-            failf(data, "Receeived problem in the chunky parser");
-            return CURLE_READ_ERROR;
+            if(CHUNKE_WRITE_ERROR == res) {
+              failf(data, "Failed writing data");
+              return CURLE_WRITE_ERROR;
+            }
+            failf(data, "Received problem in the chunky parser");
+            return CURLE_RECV_ERROR;
           }
           else if(CHUNKE_STOP == res) {
             /* we're done reading chunks! */
@@ -722,8 +754,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
         conn->upload_fromhere = k->uploadbuf;
 
         nread = data->set.fread(conn->upload_fromhere, 1,
-                                conn->upload_bufsize,
-                                data->set.in);
+                                BUFSIZE, data->set.in);
 
         /* the signed int typecase of nread of for systems that has
            unsigned size_t */
@@ -740,22 +771,29 @@ CURLcode Curl_readwrite(struct connectdata *conn,
         /* convert LF to CRLF if so asked */
         if (data->set.crlf) {
           for(i = 0, si = 0; i < nread; i++, si++) {
-            if (k->buf[i] == 0x0a) {
+            if (conn->upload_fromhere[i] == 0x0a) {
               data->state.scratch[si++] = 0x0d;
               data->state.scratch[si] = 0x0a;
             }
-            else {
-              data->state.scratch[si] = k->uploadbuf[i];
-            }
+            else
+              data->state.scratch[si] = conn->upload_fromhere[i];
+          }
+          if(si != nread) {
+            /* only perform the special operation if we really did replace
+               anything */
+            nread = si;
+
+            /* upload from the new (replaced) buffer instead */
+            conn->upload_fromhere = data->state.scratch;
+
+            /* set the new amount too */
+            conn->upload_present = nread;
           }
-          nread = si;
-          k->buf = data->state.scratch; /* point to the new buffer */
         }
       }
       else {
         /* We have a partial buffer left from a previous "round". Use
            that instead of reading more data */
-
       }
 
       /* write to socket */
@@ -776,15 +814,13 @@ CURLcode Curl_readwrite(struct connectdata *conn,
            is to happen */
         conn->upload_fromhere += bytes_written;
       }
-      else if(data->set.crlf)
-        k->buf = data->state.buffer; /* put it back on the buffer */
       else {
         /* we've uploaded that buffer now */
         conn->upload_fromhere = k->uploadbuf;
         conn->upload_present = 0; /* no more bytes left */
       }
 
-      k->writebytecount += nread;
+      k->writebytecount += bytes_written;
       Curl_pgrsSetUploadCounter(data, (double)k->writebytecount);
 
     }
@@ -819,15 +855,6 @@ CURLcode Curl_readwrite(struct connectdata *conn,
   if (result)
     return result;
     
-  if(data->progress.ulspeed > conn->upload_bufsize) {
-    /* If we're transfering more data per second than fits in our buffer,
-       we increase the buffer size to adjust to the current
-       speed. However, we must not set it larger than BUFSIZE. We don't
-       adjust it downwards again since we don't see any point in that!
-    */
-    conn->upload_bufsize=(long)min(data->progress.ulspeed, BUFSIZE);
-  }
-
   if (data->set.timeout &&
       ((Curl_tvdiff(k->now, k->start)/1000) >= data->set.timeout)) {
     failf (data, "Operation timed out with %d out of %d bytes received",
@@ -981,7 +1008,7 @@ Transfer(struct connectdata *conn)
     k->writefd = k->wkeepfd;
     interval.tv_sec = 1;
     interval.tv_usec = 0;
-    
+
     switch (select (k->maxfd, &k->readfd, &k->writefd, NULL,
                     &interval)) {
     case -1: /* select() error, stop reading */
@@ -997,6 +1024,7 @@ Transfer(struct connectdata *conn)
     case 0:  /* timeout */
       result = Curl_readwrite(conn, &done);
       break;
+
     default: /* readable descriptors */
       result = Curl_readwrite(conn, &done);
       break;
