@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: main.c,v 1.107 2002/01/03 08:07:29 bagder Exp $
+ * $Id: main.c,v 1.124 2002/04/08 22:48:25 bagder Exp $
  *****************************************************************************/
 
 /* This is now designed to have its own local setup.h */
@@ -39,6 +39,9 @@
 
 #include "urlglob.h"
 #include "writeout.h"
+#ifdef USE_ENVIRONMENT
+#include "writeenv.h"
+#endif
 
 #define CURLseparator	"--_curl_--"
 
@@ -191,6 +194,11 @@ globalvalue int  CURL_BADPARAM;
 globalvalue int  CURL_BADORDER;
 globalvalue int  CURL_BADPWD;
 globalvalue int  CURL_MNYREDIR;
+globalvalue int  CURL_UNKTELNET;
+globalvalue int  CURL_UNKMSG;
+globalvalue int  CURL_BADSSLCERT;
+globalvalue int  CURL_SRVNOERR;
+globalvalue int  CURL_MAXMSG;
 long	vms_cond[] = {
 	VMS_STS(1,FAC_SYSTEM,MSG_NORMAL,SEV_SUCCESS),
 	CURL_UNSUPPROTO,		/* these are from curlmsg.msg file..... */
@@ -238,7 +246,12 @@ long	vms_cond[] = {
 	CURL_BADPARAM,
 	CURL_BADORDER,
 	CURL_BADPWD,
-	CURL_MNYREDIR
+	CURL_MNYREDIR,
+	CURL_UNKTELNET,
+	CURL_UNKMSG,
+	CURL_BADSSLCERT,
+	CURL_SRVNOERR,
+	CURL_MAXMSG
 };
 #endif
 
@@ -318,6 +331,9 @@ static void help(void)
        "    --disable-epsv  Prevents curl from using EPSV (F)\n"
        " -D/--dump-header <file> Write the headers to this file\n"
        "    --egd-file <file> EGD socket path for random data (SSL)\n"
+#ifdef USE_ENVIRONMENT
+       "    --environment   Write result codes to environment variables (RISC OS)\n"
+#endif
        " -e/--referer       Referer page (H)");
   puts(" -E/--cert <cert[:passwd]> Specifies your certificate file and password (HTTPS)\n"
        "    --cert-type <type> Specifies your certificate file type (DER/PEM/ENG) (HTTPS)\n"
@@ -443,11 +459,14 @@ struct Configurable {
   bool use_httpget;
 
   char *writeout; /* %-styled format string to output */
+  bool writeenv; /* write results to environment, if available */
 
   FILE *errors; /* if stderr redirect is requested */
+  bool errors_fopened;
 
   struct curl_slist *quote;
   struct curl_slist *postquote;
+  struct curl_slist *prequote;
 
   long ssl_version;
   curl_TimeCond timecond;
@@ -455,15 +474,15 @@ struct Configurable {
 
   struct curl_slist *headers;
 
-  struct HttpPost *httppost;
-  struct HttpPost *last_post;
+  struct curl_httppost *httppost;
+  struct curl_httppost *last_post;
 
   struct curl_slist *telnet_options;
         
   HttpReq httpreq;
 };
 
-static int parseconfig(char *filename,
+static int parseconfig(const char *filename,
 		       struct Configurable *config);
 static char *my_get_line(FILE *fp);
 
@@ -586,13 +605,16 @@ struct multi_files {
 
 /* Add a new list entry possibly with a type_name
  */
-static void *AddMultiFiles (const char *file_name,
-                            const char *type_name,
-                            struct multi_files **multi_start,
-                            struct multi_files **multi_current)
+static struct multi_files *
+AddMultiFiles (const char *file_name,
+               const char *type_name,
+               const char *show_filename,
+               struct multi_files **multi_start,
+               struct multi_files **multi_current)
 {
   struct multi_files *multi;
   struct multi_files *multi_type = NULL;
+  struct multi_files *multi_name = NULL;
   multi = (struct multi_files *)malloc(sizeof(struct multi_files));
   if (multi) {
     memset(multi, 0, sizeof(struct multi_files));
@@ -601,6 +623,10 @@ static void *AddMultiFiles (const char *file_name,
   }
   else
     return NULL;
+
+  if (!*multi_start)
+    *multi_start = multi;
+
   if (type_name) {
     multi_type = (struct multi_files *)malloc(sizeof(struct multi_files));
     if (multi_type) {
@@ -608,41 +634,46 @@ static void *AddMultiFiles (const char *file_name,
       multi_type->form.option = CURLFORM_CONTENTTYPE;
       multi_type->form.value = type_name;
       multi->next = multi_type;
+
+      multi = multi_type;
     }
     else {
       free (multi);
       return NULL;
     }
   }
-  if (!*multi_start)
-    *multi_start = multi;
-  if (!*multi_current) {
-    if (multi_type)
-      *multi_current = multi_type;
-    else
-      *multi_current = multi;
-  }
-  else {
-    if (multi_type) {
-      (*multi_current)->next = multi;
-      *multi_current = multi_type;
+  if (show_filename) {
+    multi_name = (struct multi_files *)malloc(sizeof(struct multi_files));
+    if (multi_name) {
+      memset(multi_name, 0, sizeof(struct multi_files));
+      multi_name->form.option = CURLFORM_FILENAME;
+      multi_name->form.value = show_filename;
+      multi->next = multi_name;
+
+      multi = multi_name;
     }
     else {
-      (*multi_current)->next = multi;
-      *multi_current = multi;
+      free (multi);
+      return NULL;
     }
   }
+
+  if (*multi_current)
+    (*multi_current)->next = multi;
+
+  *multi_current = multi;
+
   return *multi_current;
 }
 
 /* Free the items of the list.
  */
-static void FreeMultiInfo (struct multi_files **multi_start)
+static void FreeMultiInfo (struct multi_files *multi_start)
 {
   struct multi_files *multi;
-  while (*multi_start) {
-    multi = *multi_start;
-    *multi_start = (*multi_start)->next;
+  while (multi_start) {
+    multi = multi_start;
+    multi_start = multi_start->next;
     free (multi);
   }
 }
@@ -665,8 +696,19 @@ static void FreeMultiInfo (struct multi_files **multi_start)
  *
  * 'name=@filename;type=image/gif,filename2,filename3'
  *
- * Does use curl_formadd to fulfill it's job. Is heavily based on the
- * old curl_formparse code.
+ * If you want custom headers added for a single part, write them in a separate
+ * file and do like this:
+ *
+ * 'name=foo;headers=@headerfile' or why not
+ * 'name=@filemame;headers=@headerfile'
+ *
+ * To upload a file, but to fake the file name that will be included in the
+ * formpost, do like this:
+ *
+ * 'name=@filename;filename=/dev/null'
+ *
+ * This function uses curl_formadd to fulfill it's job. Is heavily based on
+ * the old curl_formparse code.
  *
  ***************************************************************************/
 
@@ -674,8 +716,8 @@ static void FreeMultiInfo (struct multi_files **multi_start)
 #define FORM_TYPE_SEPARATOR ';'
 
 static int formparse(char *input,
-                     struct HttpPost **httppost,
-                     struct HttpPost **last_post)
+                     struct curl_httppost **httppost,
+                     struct curl_httppost **last_post)
 {
   /* nextarg MUST be a string in the format 'name=contents' and we'll
      build a linked list with the info */
@@ -706,7 +748,9 @@ static int formparse(char *input,
 
       do {
 	/* since this was a file, it may have a content-type specifier
-	   at the end too */
+	   at the end too, or a filename. Or both. */
+        char *ptr;
+        char *filename=NULL;
 
 	sep=strchr(contp, FORM_TYPE_SEPARATOR);
 	sep2=strchr(contp, FORM_FILE_SEPARATOR);
@@ -717,35 +761,67 @@ static int formparse(char *input,
 
 	  /* no type was specified! */
 	}
+
+        type = NULL;
+
 	if(sep) {
 
 	  /* if we got here on a comma, don't do much */
-	  if(FORM_FILE_SEPARATOR != *sep)
-	    type = strstr(sep+1, "type=");
+	  if(FORM_FILE_SEPARATOR == *sep)
+	    ptr = NULL;
 	  else
-	    type=NULL;
+            ptr = sep+1;
 
 	  *sep=0; /* terminate file name at separator */
 
-	  if(type) {
-	    type += strlen("type=");
-	    
-	    if(2 != sscanf(type, "%127[^/]/%127[^,\n]",
-			   major, minor)) {
-	      fprintf(stderr, "Illegally formatted content-type field!\n");
-              free(contents);
-              FreeMultiInfo (&multi_start);
-	      return 2; /* illegal content-type syntax! */
-	    }
-	    /* now point beyond the content-type specifier */
-	    sep = (char *)type + strlen(major)+strlen(minor)+1;
+	  while(ptr && (FORM_FILE_SEPARATOR!= *ptr)) {
 
-	    /* find the following comma */
-	    sep=strchr(sep, FORM_FILE_SEPARATOR);
+            /* pass all white spaces */
+            while(isspace((int)*ptr))
+              ptr++;
+
+            if(curl_strnequal("type=", ptr, 5)) {
+
+              /* set type pointer */
+              type = &ptr[5];
+	    
+              /* verify that this is a fine type specifier */
+              if(2 != sscanf(type, "%127[^/]/%127[^;,\n]",
+                             major, minor)) {
+                fprintf(stderr, "Illegally formatted content-type field!\n");
+                free(contents);
+                FreeMultiInfo (multi_start);
+                return 2; /* illegal content-type syntax! */
+              }
+              /* now point beyond the content-type specifier */
+              sep = (char *)type + strlen(major)+strlen(minor)+1;
+
+              *sep=0; /* zero terminate type string */
+
+              ptr=sep+1;
+            }
+            else if(curl_strnequal("filename=", ptr, 9)) {
+              filename = &ptr[9];
+              ptr=strchr(filename, FORM_TYPE_SEPARATOR);
+              if(!ptr) {
+                ptr=strchr(filename, FORM_FILE_SEPARATOR);
+              }
+              if(ptr) {
+                *ptr=0; /* zero terminate */
+                ptr++;
+              }
+            }
+            else
+              /* confusion, bail out of loop */
+              break;
 	  }
+          /* find the following comma */
+          if(ptr)
+            sep=strchr(ptr, FORM_FILE_SEPARATOR);
+          else
+            sep=NULL;
 	}
 	else {
-	  type=NULL;
 	  sep=strchr(contp, FORM_FILE_SEPARATOR);
 	}
 	if(sep) {
@@ -755,14 +831,17 @@ static int formparse(char *input,
 	}
         /* if type == NULL curl_formadd takes care of the problem */
 
-        if (!AddMultiFiles (contp, type, &multi_start, &multi_current)) {
+        if (!AddMultiFiles (contp, type, filename, &multi_start,
+                            &multi_current)) {
           fprintf(stderr, "Error building form post!\n");
           free(contents);
-          FreeMultiInfo (&multi_start);
+          FreeMultiInfo (multi_start);
           return 3;
         }
 	contp = sep; /* move the contents pointer to after the separator */
+
       } while(sep && *sep); /* loop if there's another file name */
+
       /* now we add the multiple files section */
       if (multi_start) {
         struct curl_forms *forms = NULL;
@@ -778,7 +857,7 @@ static int formparse(char *input,
         {
           fprintf(stderr, "Error building form post!\n");
           free(contents);
-          FreeMultiInfo (&multi_start);
+          FreeMultiInfo (multi_start);
           return 4;
         }
         for (i = 0, ptr = multi_start; i < count; ++i, ptr = ptr->next)
@@ -787,7 +866,7 @@ static int formparse(char *input,
           forms[i].value = ptr->form.value;
         }
         forms[count].option = CURLFORM_END;
-        FreeMultiInfo (&multi_start);
+        FreeMultiInfo (multi_start);
         if (curl_formadd (httppost, last_post,
                           CURLFORM_COPYNAME, name,
                           CURLFORM_ARRAY, forms, CURLFORM_END) != 0) {
@@ -870,13 +949,17 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     {"8", "stderr",      TRUE},
     {"7", "interface",   TRUE},
     {"6", "krb4",        TRUE},
+    /* all these ones, starting with 5 as a short-option have *no* short
+       option to mention. */
     {"5", "url",         TRUE},
     {"5a", "random-file", TRUE},
     {"5b", "egd-file",   TRUE},
     {"5c", "connect-timeout", TRUE},
     {"5d", "ciphers",    TRUE},
     {"5e", "disable-epsv", FALSE},
-
+#ifdef USE_ENVIRONMENT
+    {"5f", "environment", FALSE},
+#endif
     {"0", "http1.0",     FALSE},
     {"1", "tlsv1",       FALSE},
     {"2", "sslv2",       FALSE},
@@ -1020,8 +1103,10 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       config->crlf = TRUE;
       break;
     case '8': /* there is no short letter for this */
-      if(strcmp(nextarg, "-"))
+      if(strcmp(nextarg, "-")) {
         config->errors = fopen(nextarg, "wt");
+        config->errors_fopened = TRUE;
+      }
       else
         config->errors = stdout;
       break;
@@ -1050,6 +1135,11 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       case 'e': /* --disable-epsv */
         config->disable_epsv ^= TRUE;
         break;
+#ifdef USE_ENVIRONMENT
+      case 'f':
+        config->writeenv ^= TRUE;
+        break;
+#endif
       default: /* the URL! */
         {
           struct getout *url;
@@ -1151,7 +1241,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
           if(strequal("-", nextarg))
             file = stdin;
           else 
-            file = fopen(nextarg, "r");
+            file = fopen(nextarg, "rb");
 
           if(subletter == 'b') /* forced binary */
             postdata = file2memory(file, &config->postfieldsize);
@@ -1175,9 +1265,13 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
         else
           config->postfields=postdata;
       }
+      /*
+        We can't set the request type here, as this data might be used in
+        a simple GET if -G is used. Already or soon.
 
-/*      if(SetHTTPrequest(HTTPREQ_SIMPLEPOST, &config->httpreq))
-        return PARAM_BAD_USE;*/
+        if(SetHTTPrequest(HTTPREQ_SIMPLEPOST, &config->httpreq))
+          return PARAM_BAD_USE;
+      */
       break;
     case 'D':
       /* dump-header to given file name */
@@ -1376,12 +1470,18 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       break;
     case 'Q':
       /* QUOTE command to send to FTP server */
-      if(nextarg[0] == '-') {
+      switch(nextarg[0]) {
+      case '-':
         /* prefixed with a dash makes it a POST TRANSFER one */
         nextarg++;
         config->postquote = curl_slist_append(config->postquote, nextarg);
-      }
-      else {
+        break;
+      case '+':
+        /* prefixed with a plus makes it a just-before-transfer one */
+        nextarg++;
+        config->prequote = curl_slist_append(config->prequote, nextarg);
+        break;
+      default:
         config->quote = curl_slist_append(config->quote, nextarg);
       }
       break;
@@ -1450,10 +1550,8 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       GetStr(&config->proxy, nextarg);
       break;
     case 'X':
-      /* HTTP request */
+      /* set custom request */
       GetStr(&config->customrequest, nextarg);
-      if(SetHTTPrequest(HTTPREQ_CUSTOM, &config->httpreq))
-        return PARAM_BAD_USE;
       break;
     case 'y':
       /* low speed time */
@@ -1517,31 +1615,32 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
 }
 
 
-static int parseconfig(char *filename,
+static int parseconfig(const char *filename,
 		       struct Configurable *config)
 {
   int res;
   FILE *file;
-  char filebuffer[256];
+  char filebuffer[512];
   bool usedarg;
-  char *home=NULL;
+  char *home;
   
   if(!filename || !*filename) {
     /* NULL or no file name attempts to load .curlrc from the homedir! */
 
 #define CURLRC DOT_CHAR "curlrc"
 
+    filename = CURLRC;          /* sensible default */
     home = curl_getenv("HOME"); /* portable environment reader */
-    if(!home)
-      return 0;
-    if(strlen(home)>(sizeof(filebuffer)-strlen(CURLRC))) {
-      free(home);
-      return 0;
+    if(home) {
+      if(strlen(home)<(sizeof(filebuffer)-strlen(CURLRC))) {
+
+        snprintf(filebuffer, sizeof(filebuffer),
+                 "%s%s%s", home, DIR_CHAR, CURLRC);
+
+        filename = filebuffer;
+      }
+      free(home); /* we've used it, now free it */
     }
-
-    sprintf(filebuffer, "%s%s%s", home, DIR_CHAR, CURLRC);
-
-    filename = filebuffer;
   }
 
   if(strcmp(filename,"-"))
@@ -1688,8 +1787,6 @@ static int parseconfig(char *filename,
     if(file != stdin)
       fclose(file);
   }
-  if(home)
-    free(home);
   return 0;
 }
 
@@ -1718,18 +1815,19 @@ int my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream)
 }
 
 struct ProgressData {
-  size_t total;
-  size_t prev;
-  size_t point;
+  int calls;
+  double total;
+  double prev;
+  double point;
   int width;
   FILE *out; /* where to write everything to */
 };
 
 int myprogress (void *clientp,
-                size_t dltotal,
-                size_t dlnow,
-                size_t ultotal,
-                size_t ulnow)
+                double dltotal,
+                double dlnow,
+                double ultotal,
+                double ulnow)
 {
   /* The original progress-bar source code was written for curl by Lars Aas,
      and this new edition inherites some of his concepts. */
@@ -1748,6 +1846,8 @@ int myprogress (void *clientp,
 
   bar->point = dlnow + ulnow; /* we've come this far */
 
+  bar->calls++; /* simply count invokes */
+
   if(0 == total) {
     int prevblock = bar->prev / 1024;
     int thisblock = bar->point / 1024;
@@ -1892,13 +1992,18 @@ operate(struct Configurable *config, int argc, char *argv[])
   int res = 0;
   int i;
 
-  errorbuffer[0]=0; /* prevent junk from being output */
-
 #ifdef MALLOCDEBUG
   /* this sends all memory debug messages to a logfile named memdump */
-  curl_memdebug("memdump");
+  char *env;
+  env = curl_getenv("CURL_MEMDEBUG");
+  if(env) {
+    free(env);
+    curl_memdebug("memdump");
+  }
 #endif
 
+  errorbuffer[0]=0; /* prevent junk from being output */
+
   main_init(); /* inits */
 
   config->showerror=TRUE;
@@ -2383,9 +2488,20 @@ operate(struct Configurable *config, int argc, char *argv[])
       
       res = curl_easy_perform(curl);
         
+      if((config->progressmode == CURL_PROGRESS_BAR) &&
+         progressbar.calls) {
+        /* if the custom progress bar has been displayed, we output a
+           newline here */
+        fputs("\n", progressbar.out);
+      }
+
       if(config->writeout) {
         ourWriteOut(curl, config->writeout);
       }
+#ifdef USE_ENVIRONMENT
+      if (config->writeenv)
+        ourWriteEnv(curl);
+#endif
 
 #ifdef	VMS
       if (!config->showerror)  {
@@ -2458,9 +2574,7 @@ operate(struct Configurable *config, int argc, char *argv[])
   /* cleanup the curl handle! */
   curl_easy_cleanup(curl);
 
-  if((config->errors != stderr) &&
-     (config->errors != stdout))
-    /* it wasn't directed to stdout or stderr so close the file! */
+  if(config->errors_fopened)
     fclose(config->errors);
 
   main_free(); /* cleanup */
@@ -2479,6 +2593,7 @@ int main(int argc, char *argv[])
   free_config_fields(&config);
 
 #ifdef	VMS
+  if (res > 52) res = 53;		/* MAXMSG, porting helper */
   return (vms_cond[res]|vms_show);
 #else
   return res;
