@@ -1,5 +1,5 @@
-#!/usr/bin/perl
-# $Id: runtests.pl,v 1.45 2002/01/08 09:32:41 bagder Exp $
+#!/usr/bin/env perl
+# $Id: runtests.pl,v 1.58 2002/04/09 14:28:16 bagder Exp $
 #
 # Main curl test script, in perl to run on more platforms
 #
@@ -52,7 +52,7 @@ my $memory_debug=0;
 my $memdump="memdump";
 
 # the path to the script that analyzes the memory debug output file:
-my $memanalyze="../memanalyze.pl";
+my $memanalyze="./memanalyze.pl";
 
 my $checkstunnel = &checkstunnel;
 
@@ -74,6 +74,10 @@ my $pwd;          # current working directory
 
 chomp($pwd = `pwd`);
 
+# enable memory debugging if curl is compiled with it
+$ENV{'CURL_MEMDEBUG'} = 1;
+$ENV{'HOME'}=$pwd;
+
 #######################################################################
 # Return the pid of the server as found in the given pid file
 #
@@ -140,16 +144,27 @@ sub runhttpserver {
     my $RUNNING;
     my $pid;
 
+    $pid = checkserver ($HTTPPIDFILE);
+
     # verify if our/any server is running on this port
     my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver`;
 
     if ( $data =~ /WE ROOLZ(: |)(\d*)/ ) {
-        $pid = 0+$2;
+        if($2) {
+            $pid = 0+$2;
+        }
+
+        if(!$pid) {
+            print "Test server already running with unknown pid! Use it...\n";
+            return;
+        }
+
         if($verbose) {
             print "Test server already running with pid $pid, killing it...\n";
         }
     }
     elsif($data ne "") {
+        print "GOT: $data\n";
         print "An alien HTTP server is running on port $HOSTPORT\n",
         "Edit runtests.pl to use another port and rerun the test script\n";
         exit;
@@ -160,7 +175,7 @@ sub runhttpserver {
         }
     }
 
-    if($pid) {
+    if($pid > 0) {
         my $res = kill (9, $pid); # die!
         if(!$res) {
             print "Failed to kill our HTTP test server, do it manually and",
@@ -602,6 +617,15 @@ sub singletest {
 
         my @protstrip=@protocol;
 
+        # check if there's any attributes on the verify/protocol section
+        my %hash = getpartattr("verify", "protocol");
+
+        if($hash{'nonewline'}) {
+            # Yes, we must cut off the final newline from the final line
+            # of the protocol data
+            chomp($protstrip[$#protstrip]);
+        }
+
         for(@strip) {
             # strip all patterns from both arrays
             @out = striparray( $_, \@out);
@@ -835,7 +859,6 @@ if($testthis[0] ne "") {
     $TESTCASES=join(" ", @testthis);
 }
 
-
 #######################################################################
 # Output curl version and host info being tested
 #
